<?php
/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Admin.php 4057 2006-01-12 16:21:59Z roh@m3.net $
*/

require_once MAX_PATH.'/lib/max/Dal/Common.php';
require_once 'Date.php';
require_once(MAX_PATH . '/plugins/statistics/targeting/TargetingStatistics.php');

/**
 * The non-DB specific Data Access Layer (DAL) class for the User Interface (Reporting).
 *
 * @package    MaxDal
 * @author     Andrew Hill <andrew@m3.net>
 * @author     Robert Hunter <roh@m3.net>
 */
class MAX_Dal_Reporting extends MAX_Dal_Common
{
    function MAX_Dal_Reporting()
    {
        parent :: MAX_Dal_Common();
    }

    function getPublisherDailyStats($agencyId, $publisherId, $statsStartDate = null, $statsEndDate = null)
    {
        $conf = & $GLOBALS['_MAX']['CONF'];
        
        $adminConstraint = $agencyId > 0 ? "AND a.agencyid=$agencyId" : '';
        $affiliateConstraint = $publisherId > 0 ? "AND z.affiliateid=$publisherId" : '';
        $statsStartConstraint = !empty($statsStartDate) ? "AND dsah.day>='$statsStartDate'" : '';
        $statsEndConstraint = !empty($statsEndDate) ? "AND dsah.day<='$statsEndDate'" : '';
        
        $aAdvertiserDailyStatsData = array();
        
        // 3.  Get the TOTAL statistics for these campaigns
        $query = "
        SELECT
            dsah.day AS day,
            SUM(dsah.impressions) AS impressions,
            SUM(dsah.clicks) AS clicks
        FROM
            {$conf['table']['clients']} AS a,
            {$conf['table']['campaigns']} AS c,
            {$conf['table']['banners']} AS b,
            {$conf['table']['data_summary_ad_hourly']} AS dsah
            LEFT JOIN {$conf['table']['zones']} AS z ON dsah.zone_id=z.zoneid
        WHERE
            c.clientid=a.clientid
            AND b.campaignid=c.campaignid
            AND dsah.ad_id=b.bannerid
            $statsStartConstraint
            $statsEndConstraint
            $adminConstraint
            $affiliateConstraint
        GROUP BY
            day
        ";
        
        $res = phpAds_dbQuery($query) or phpAds_sqlDie();
        
        while ($row = phpAds_dbFetchArray($res)) {
            $aAdvertiserDailyStatsData[$row['day']]['day'] = $row['day'];
            $aAdvertiserDailyStatsData[$row['day']]['impressions'] = $row['impressions'];
            $aAdvertiserDailyStatsData[$row['day']]['clicks'] = $row['clicks'];
        }
        
        return $aAdvertiserDailyStatsData;
        
    }
    function getCampaignData($agencyId, $publisherId, $campaignStartDate, $campaignEndDate, $statsStartDate = null, $statsEndDate = null)
    {
        $conf = & $GLOBALS['_MAX']['CONF'];
        
        $adminConstraint = $agencyId > 0 ? "AND a.agencyid=$agencyId" : '';
        $affiliateConstraint = $publisherId > 0 ? "AND z.affiliateid=$publisherId" : '';
        $campaignStartConstraint = !empty($campaignStartDate) ? "AND (c.expire>='$campaignStartDate' OR c.expire='0000-00-00')" : '';
        $campaignEndConstraint = !empty($campaignEndDate) ? "AND (c.activate<='$campaignEndDate' OR c.activate='0000-00-00')" : '';
        $statsStartConstraint = !empty($statsStartDate) ? "AND dsah.day>='$statsStartDate'" : '';
        $statsEndConstraint = !empty($statsEndDate) ? "AND dsah.day<='$statsEndDate'" : '';
        
        $aCampaignData = array();
        
        // 1. Get the list of campaigns that are linked to zones
        $query = "
        SELECT
            c.campaignid AS campaign_id,
            c.campaignname AS campaign_name,
            c.active AS campaign_active,
            IF(c.priority='-1','1-exclusive',IF(c.priority='0','3-low','2-high')) AS campaign_priority,
            c.activate AS campaign_start_date,
            c.expire AS campaign_end_date,
            TO_DAYS(c.activate) AS campaign_start_date_days,
            TO_DAYS(c.expire) AS campaign_end_date_days,
            c.views AS campaign_booked_impressions,
            z.zoneid AS zone_id,
            z.zonename AS zone_name,
            z.width AS zone_width,
            z.height AS zone_height
        FROM
            {$conf['table']['clients']} AS a,
            {$conf['table']['campaigns']} AS c
            LEFT JOIN {$conf['table']['placement_zone_assoc']} AS pza ON c.campaignid=pza.placement_id
            LEFT JOIN {$conf['table']['zones']} AS z ON pza.zone_id=z.zoneid
        WHERE
            c.clientid=a.clientid
            $campaignStartConstraint
            $campaignEndConstraint
            $adminConstraint
            $affiliateConstraint
        ";
        
        $res = phpAds_dbQuery($query) or phpAds_sqlDie();
        
        while ($row = phpAds_dbFetchArray($res)) {
            if (empty($aCampaignData[$row['campaign_id']])) {
                $aCampaignData[$row['campaign_id']]['campaign_id'] = $row['campaign_id'];
                $aCampaignData[$row['campaign_id']]['campaign_name'] = $row['campaign_name'];
                $aCampaignData[$row['campaign_id']]['campaign_active'] = $row['campaign_active'];
                $aCampaignData[$row['campaign_id']]['campaign_priority'] = $row['campaign_priority'];
                $aCampaignData[$row['campaign_id']]['campaign_start_date'] = $row['campaign_start_date'];
                $aCampaignData[$row['campaign_id']]['campaign_end_date'] = $row['campaign_end_date'];
                $aCampaignData[$row['campaign_id']]['campaign_start_date_days'] = $row['campaign_start_date_days'];
                $aCampaignData[$row['campaign_id']]['campaign_end_date_days'] = $row['campaign_end_date_days'];
                $aCampaignData[$row['campaign_id']]['campaign_booked_impressions'] = $row['campaign_booked_impressions'];
                $aCampaignData[$row['campaign_id']]['placement_zones'] = array();
                $aCampaignData[$row['campaign_id']]['ad_zones'] = array();
                $aCampaignData[$row['campaign_id']]['stats_zones'] = array();
            }

            if (!empty($row['zone_id']) && empty($aCampaignData[$row['campaign_id']]['placement_zones'][$row['zone_id']])) {
                $aCampaignData[$row['campaign_id']]['placement_zones'][$row['zone_id']]['zone_id'] = $row['zone_id'];
                $aCampaignData[$row['campaign_id']]['placement_zones'][$row['zone_id']]['zone_name'] = $row['zone_name'];
                $aCampaignData[$row['campaign_id']]['placement_zones'][$row['zone_id']]['zone_width'] = $row['zone_width'];
                $aCampaignData[$row['campaign_id']]['placement_zones'][$row['zone_id']]['zone_height'] = $row['zone_height'];
            }
        }
        
        // 2.  Get the list of campaigns whose banners are linked to zones
        $query = "
        SELECT
            c.campaignid AS campaign_id,
            c.campaignname AS campaign_name,
            c.active AS campaign_active,
            IF(c.priority='-1','1-exclusive',IF(c.priority='0','3-low','2-high')) AS campaign_priority,
            c.activate AS campaign_start_date,
            c.expire AS campaign_end_date,
            TO_DAYS(c.activate) AS campaign_start_date_days,
            TO_DAYS(c.expire) AS campaign_end_date_days,
            c.views AS campaign_booked_impressions,
            z.zoneid AS zone_id,
            z.zonename AS zone_name,
            z.width AS zone_width,
            z.height AS zone_height
        FROM
            {$conf['table']['clients']} AS a,
            {$conf['table']['campaigns']} AS c
            LEFT JOIN {$conf['table']['banners']} AS b ON c.campaignid=b.campaignid
            LEFT JOIN {$conf['table']['ad_zone_assoc']} AS aza ON b.bannerid=aza.ad_id
            LEFT JOIN {$conf['table']['zones']} AS z ON aza.zone_id=z.zoneid
        WHERE
            c.clientid=a.clientid
            $campaignStartConstraint
            $campaignEndConstraint
            $adminConstraint
            $affiliateConstraint
        ";
        
        $res = phpAds_dbQuery($query) or phpAds_sqlDie();
        
        while ($row = phpAds_dbFetchArray($res)) {
            if (empty($aCampaignData[$row['campaign_id']])) {
                $aCampaignData[$row['campaign_id']]['campaign_id'] = $row['campaign_id'];
                $aCampaignData[$row['campaign_id']]['campaign_name'] = $row['campaign_name'];
                $aCampaignData[$row['campaign_id']]['campaign_active'] = $row['campaign_active'];
                $aCampaignData[$row['campaign_id']]['campaign_priority'] = $row['campaign_priority'];
                $aCampaignData[$row['campaign_id']]['campaign_start_date'] = $row['campaign_start_date'];
                $aCampaignData[$row['campaign_id']]['campaign_end_date'] = $row['campaign_end_date'];
                $aCampaignData[$row['campaign_id']]['campaign_start_date_days'] = $row['campaign_start_date_days'];
                $aCampaignData[$row['campaign_id']]['campaign_end_date_days'] = $row['campaign_end_date_days'];
                $aCampaignData[$row['campaign_id']]['campaign_booked_impressions'] = $row['campaign_booked_impressions'];
                $aCampaignData[$row['campaign_id']]['placement_zones'] = array();
                $aCampaignData[$row['campaign_id']]['ad_zones'] = array();
                $aCampaignData[$row['campaign_id']]['stats_zones'] = array();
            }

            if (!empty($row['zone_id']) && empty($aCampaignData[$row['campaign_id']]['ad_zones'][$row['zone_id']])) {
                $aCampaignData[$row['campaign_id']]['ad_zones'][$row['zone_id']]['zone_id'] = $row['zone_id'];
                $aCampaignData[$row['campaign_id']]['ad_zones'][$row['zone_id']]['zone_name'] = $row['zone_name'];
                $aCampaignData[$row['campaign_id']]['ad_zones'][$row['zone_id']]['zone_width'] = $row['zone_width'];
                $aCampaignData[$row['campaign_id']]['ad_zones'][$row['zone_id']]['zone_height'] = $row['zone_height'];
            }
        }
        
        // 3.  Get the TOTAL statistics for these campaigns
        $query = "
        SELECT
            c.campaignid AS campaign_id,
            c.campaignname AS campaign_name,
            c.active AS campaign_active,
            IF(c.priority='-1','1-exclusive',IF(c.priority='0','3-low','2-high')) AS campaign_priority,
            c.activate AS campaign_start_date,
            c.expire AS campaign_end_date,
            TO_DAYS(c.activate) AS campaign_start_date_days,
            TO_DAYS(c.expire) AS campaign_end_date_days,
            c.views AS campaign_booked_impressions,
            z.zoneid AS zone_id,
            z.zonename AS zone_name,
            z.width AS zone_width,
            z.height AS zone_height,
            SUM(dsah.impressions) AS campaign_impressions,
            SUM(dsah.clicks) AS campaign_clicks
        FROM
            {$conf['table']['clients']} AS a,
            {$conf['table']['campaigns']} AS c,
            {$conf['table']['banners']} AS b,
            {$conf['table']['data_summary_ad_hourly']} AS dsah
            LEFT JOIN {$conf['table']['zones']} AS z ON dsah.zone_id=z.zoneid
        WHERE
            c.clientid=a.clientid
            AND b.campaignid=c.campaignid
            AND dsah.ad_id=b.bannerid
            $statsStartConstraint
            $statsEndConstraint
            $adminConstraint
            $affiliateConstraint
        GROUP BY
            campaign_id,
            zone_id
        ";
        
        $res = phpAds_dbQuery($query) or phpAds_sqlDie();
        
        while ($row = phpAds_dbFetchArray($res)) {
            if (empty($aCampaignData[$row['campaign_id']])) {
                $aCampaignData[$row['campaign_id']]['campaign_id'] = $row['campaign_id'];
                $aCampaignData[$row['campaign_id']]['campaign_name'] = $row['campaign_name'];
                $aCampaignData[$row['campaign_id']]['campaign_active'] = $row['campaign_active'];
                $aCampaignData[$row['campaign_id']]['campaign_priority'] = $row['campaign_priority'];
                $aCampaignData[$row['campaign_id']]['campaign_start_date'] = $row['campaign_start_date'];
                $aCampaignData[$row['campaign_id']]['campaign_end_date'] = $row['campaign_end_date'];
                $aCampaignData[$row['campaign_id']]['campaign_start_date_days'] = $row['campaign_start_date_days'];
                $aCampaignData[$row['campaign_id']]['campaign_end_date_days'] = $row['campaign_end_date_days'];
                $aCampaignData[$row['campaign_id']]['campaign_booked_impressions'] = $row['campaign_booked_impressions'];
                $aCampaignData[$row['campaign_id']]['campaign_impressions'] = 0;
                $aCampaignData[$row['campaign_id']]['campaign_clicks'] = 0;
                $aCampaignData[$row['campaign_id']]['placement_zones'] = array();
                $aCampaignData[$row['campaign_id']]['ad_zones'] = array();
                $aCampaignData[$row['campaign_id']]['stats_zones'] = array();
            }
            
            if (!empty($row['zone_id']) && empty($aCampaignData[$row['campaign_id']]['stats_zones'][$row['zone_id']])) {
                $aCampaignData[$row['campaign_id']]['stats_zones'][$row['zone_id']]['zone_id'] = $row['zone_id'];
                $aCampaignData[$row['campaign_id']]['stats_zones'][$row['zone_id']]['zone_name'] = $row['zone_name'];
                $aCampaignData[$row['campaign_id']]['stats_zones'][$row['zone_id']]['zone_width'] = $row['zone_width'];
                $aCampaignData[$row['campaign_id']]['stats_zones'][$row['zone_id']]['zone_height'] = $row['zone_height'];
                $aCampaignData[$row['campaign_id']]['stats_zones'][$row['zone_id']]['zone_impressions'] = $row['campaign_impressions'];
                $aCampaignData[$row['campaign_id']]['stats_zones'][$row['zone_id']]['zone_clicks'] = $row['campaign_clicks'];
            }
            
            $aCampaignData[$row['campaign_id']]['campaign_impressions'] += $row['campaign_impressions'];
            $aCampaignData[$row['campaign_id']]['campaign_clicks'] += $row['campaign_clicks'];
        }
        
        return $aCampaignData;
    }
}